'use strict';
angular.module('aom.services')
  .factory('aomAttachmentsSrv', function ($q, $ionicPopup, $timeout, aomDatabaseSrv, aomAPISrv, aomFileSystemSrv, aomMediaSrv, aomPopupSrv, aomAutoSaveSrv, aomResourcesSrv, aomLogSrv, aomJobSrv, aomLocationSrv, constants, aomTextScreenSrv) {
    var attachmentsService = {};
    var attachmentUploading = false;

    attachmentsService.data = {
      attachments: [],
      options: {},
      selectedAttachment: {}
    };

    function getAttachmentByName(name) {
      return Enumerable.from(attachmentsService.data.attachments).where('$.filename === "' + name + '"').toArray()[0];
    }

    function sendAttachment(path, attachment) {
      var def = $q.defer();

      function initUpload() {
        if (attachmentUploading) {
          $timeout(function () {
            initUpload();
          }, 5000);
        } else {
          attachmentUploading = true;
          var selectQuery = {
            attachments: { fileName: null, jobId: null, mimeType: null },
            where: {
              jobId: attachment.jobId,
              remoteHref: ';IS NULL',
              fileName: attachment.fileName
            }
          };
          aomDatabaseSrv.selectQuery(selectQuery)
            .then(function (attachmentNotSent) {
              if (attachmentNotSent.rows.length !== 0) {
                aomAPISrv.fileUpload(path + '/', attachment.fileName, attachment.mimeType, null, attachment.timestamp)
                  .then(function (location) {
                    var updateQuery = {
                      Attachments: {
                        jobId: attachment.jobId,
                        fileName: attachment.fileName,
                        remoteHref: location,
                        mimeType: attachment.mimeType
                      }
                    };
                    return aomDatabaseSrv.insertQuery(updateQuery, true, false);
                  })
                  .then(function () {
                    attachmentUploading = false;
                    def.resolve();
                  })
                  .catch(function (err) {
                    attachmentUploading = false;
                    aomLogSrv.log({ message: 'Failed to upload file.', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.warn);
                    def.reject();
                  });
              } else {
                attachmentUploading = false;
                def.resolve();
              }
            });
        }
      }

      initUpload();
      return def.promise;
    }

    function sendAttachments(attachmentsToSend) {
      var promises = [];
      for (var i = 0; i < attachmentsToSend.rows.length; i++) {
        promises.push(sendAttachment(aomFileSystemSrv.applicationDataPathNormalized, attachmentsToSend.rows[i]));
      }
      return $q.all(promises);
    }

    attachmentsService.updateJobId = function (originalId, newId, wasTemplate) {
      if (wasTemplate) {
        // TODO Update in mem
        return aomDatabaseSrv.updateQuery({ Attachments: { jobId: newId }, where: { jobId: originalId } });
      } else {
        return $q.when();
      }
    };

    attachmentsService.addAttachment = function (jobId, file, comment, source) {
      comment = comment === null || comment === undefined ? '' : comment;
      var attachment = {};
      var query = {};
      return aomLocationSrv.getAccurateCurrentLocation()
        .then(function () {
          attachment = {
            filename: file.name,
            Location: { Latitude: aomLocationSrv.data.lastKnownLocation.latitude, Longitude: aomLocationSrv.data.lastKnownLocation.longitude },
            LinkReference: file.name,
            DateCaptured: moment().format(),
            Comment: comment,
            MimeType: file.type.indexOf('/') !== -1 ? file.type : aomMediaSrv.getMediaType(file.type),
            Source: 'From Device'
          };
          query = {
            Attachments: {
              jobId: jobId,
              fileName: file.name,
              mimeType: attachment.MimeType,
              saveState: 'add',
              timestamp: aomJobSrv.data.RecordDateTimeForField && source === 'Photo' ? moment().format() : '',
              source: source
            }
          };
          return aomDatabaseSrv.insertQuery(query, true, false);
        })
        .then(function () {
          aomAutoSaveSrv.saveTempJob();
          attachmentsService.data.attachments.push(attachment);
          if (Object.keys(aomJobSrv.data.selectedField).length !== 0 && !aomJobSrv.data.selectedField.Data) {
            aomJobSrv.data.selectedField.Data = aomJobSrv.data.selectedField.Type !== constants.fieldTypes.signature ? 'Attachment Added' : aomJobSrv.data.selectedField.Attachments[0].Comment.length > 0 ? 'Attachment Added' : null;
          }
          return attachment.filename;
        });
    };

    attachmentsService.commitAttachmentUpdates = function (jobId) {
      return aomDatabaseSrv.deleteQuery({ Attachments: { jobId: jobId, saveState: 'delete' } })
        .then(function () {
          return aomDatabaseSrv.updateQuery({ Attachments: { saveState: null }, where: { jobId: jobId, saveState: 'add' } });
        });
    };


    attachmentsService.deleteAttachment = function (name) {
      var index = attachmentsService.data.attachments.indexOf(getAttachmentByName(name));

      if (index > -1) {
        var query = {
          attachments: { fileName: null, jobId: null, mimeType: null, saveState: null },
          where: {
            fileName: name
          }
        };
        return aomDatabaseSrv.selectQuery(query)
          .then(function (attachmentToDelete) {
            attachmentToDelete.rows[0].saveState = 'delete';
            return aomDatabaseSrv.insertQuery({ Attachments: attachmentToDelete.rows[0] }, true, false);
          })
          .then(function () {
            return attachmentsService.removeInMemoryAttachment(name);
          });
      } else {
        return $q.when();
      }
    };

    attachmentsService.removeInMemoryAttachment = function (name) {
      var index = attachmentsService.data.attachments.indexOf(getAttachmentByName(name));

      attachmentsService.data.attachments.splice(index, 1);
      if (Object.keys(aomJobSrv.data.selectedField).length !== 0 && attachmentsService.data.attachments.length === 0) {
        aomJobSrv.data.selectedField.Data = '';
      }
      return $q.when();
    };

    attachmentsService.deleteAttachmentFromDatabaseByJobId = function (Id) {
      var query = {
        Attachments: { jobId: Id }
      };
      return aomDatabaseSrv.deleteQuery(query);
    };

    attachmentsService.editAttachment = function (name, $scope) {
      var attachment = getAttachmentByName(name);

      $scope.selectObject = {
        DisplayName: aomResourcesSrv.data.resources.Common.EnterComment,
        Data: attachment.Comment,
        attachment: attachment
      };

      $scope.change = function () {
        $scope.selectObject.attachment.Comment = $scope.selectObject.Data;
      };

      aomTextScreenSrv.show($scope);
    };

    attachmentsService.sendAttachments = function () {
      var query = {
        attachments: { fileName: null, jobId: null, mimeType: null, timestamp: null },
        where: {
          remoteHref: ';IS NULL'
        }
      };
      return aomDatabaseSrv.selectQuery(query)
        .then(function (attachmentsToSend) {
          return sendAttachments(attachmentsToSend);
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'An error occured sending attachments', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.err);
        });
    };

    attachmentsService.sendAttachmentsForJob = function (jobId) {
      var query = {
        attachments: { fileName: null, jobId: null, mimeType: null, timestamp: null },
        where: {
          remoteHref: ';IS NULL',
          jobId: jobId
        }
      };
      return aomDatabaseSrv.selectQuery(query)
        .then(function (attachmentsToSend) {
          return sendAttachments(attachmentsToSend);
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'An error occured sending attachments', error: err }, aomLogSrv.levels.medium, aomLogSrv.severities.err);
          return $q.reject(err);
        });
    };

    attachmentsService.addLinksToJob = function (job) {
      var query = {
        attachments: { fileName: null, jobId: null, remoteHref: null },
        where: {
          jobId: job.Id
        }
      };

      return aomDatabaseSrv.selectQuery(query)
        .then(function (attachmentsOnJob) {
          attachmentsOnJob = attachmentsOnJob.rows;
          for (var i = 0; i < attachmentsOnJob.length; i++) {
            job.links.attachments[attachmentsOnJob[i].fileName] = {
              filename: attachmentsOnJob[i].fileName,
              href: attachmentsOnJob[i].remoteHref
            };
          }
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Error occured whilst adding links to job', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
        });
    };

    return attachmentsService;
  });