'use strict';
angular.module('aom.services')
  .factory('aomCustomStylesSrv', function ($http, aomResourcesSrv) {
    var customStyles = {},
      styleElementId = 'customStyles',
      userColoursStyleElementId = 'userColoursStyles';

    customStyles.init = function () {
      return $http.get('./css/style.css')
        .then(function (css) {

          var props = aomResourcesSrv.data.resources.Style.ColourScheme;
          props.SecondaryColour = { Data: props.BrandColour.Data };
          props.NavigationColour = { Data: props.BrandColour.Data };

          for (var prop in props) {
            var re = new RegExp(prop, 'g');
            var colour = aomResourcesSrv.data.resources.Style.ColourScheme[prop].Data;

            if (prop === 'SecondaryColour') colour = adjustColour(0.10, colour);
            if (prop === 'NavigationColour') colour = adjustColour(-0.10, colour);

            css.data = css.data.replace(re, colour);
          }

          var reFontSize = new RegExp('FontSize', 'g');
          css.data = css.data.replace(reFontSize, aomResourcesSrv.data.resources.appConfig.LargeFont ? '130%' : '107%');

          var reLineHeight = new RegExp('LineHeight', 'g');
          css.data = css.data.replace(reLineHeight, aomResourcesSrv.data.resources.appConfig.LargeFont ? '25px' : '20px');

          attachToDocument(css, styleElementId);
        });
    };

    function attachToDocument(css, id) {
      var styleEl = document.createElement('style');
      styleEl.id = id;
      styleEl.innerHTML = css.data;

      var existing = document.getElementById(id);
      if (existing) {
        existing.parentNode.removeChild(existing);
      }

      document.head.appendChild(styleEl);
    }

    function rgbFromHex(d) {
      var l = d.length, RGB = {};
      if (l < 6) d = '#' + d[1] + d[1] + d[2] + d[2] + d[3] + d[3] + (l > 4 ? d[4] + '' + d[4] : ''); //3 or 4 digit
      d = parseInt(d.slice(1), 16), RGB[0] = d >> 16 & 255, RGB[1] = d >> 8 & 255, RGB[2] = d & 255, RGB[3] = -1;
      if (l === 9 || l === 5) RGB[3] = Math.round((RGB[2] / 255) * 10000) / 10000, RGB[2] = RGB[1], RGB[1] = RGB[0], RGB[0] = d >> 24 & 255;
      return RGB;
    }

    function adjustColour(percentage, fromColour) {
      var isDark = percentage < 0;
      var from = rgbFromHex(fromColour);
      var to = rgbFromHex(isDark ? '#000000' : '#FFFFFF');

      percentage = isDark ? percentage * -1 : percentage;

      var hexCode = (
        0x100000000 + Math.round((to[0] - from[0]) * percentage + from[0]) *
        0x1000000 + Math.round((to[1] - from[1]) * percentage + from[1]) *
        0x10000 + Math.round((to[2] - from[2]) * percentage + from[2]) *
        0x100 + (from[3] > -1 && to[3] > -1 ? Math.round(((to[3] - from[3]) * percentage + from[3]) * 255) : to[3] > -1 ? Math.round(to[3] * 255) : from[3] > -1 ? Math.round(from[3] * 255) : 255
        )
      ).toString(16).slice(1, from[3] > -1 || to[3] > -1 ? undefined : -2);

      return '#' + hexCode;
    }

    customStyles.initUserColours = function () {
      return $http.get('./css/usercolours.css')
        .then(function (css) {
          var cssString = '';
          for (var prop in aomResourcesSrv.data.resources.Tracking.UserColours ) {
            var cssForThisColour = css.data;
            var colour = aomResourcesSrv.data.resources.Tracking.UserColours[prop];
            cssForThisColour = cssForThisColour.replace(/Code/g, prop).replace(/Hex/g, colour);
            cssString = cssString.concat(cssForThisColour, '\n');
          }

          var sheet = document.createElement('style');
          sheet.id = userColoursStyleElementId;
          sheet.innerHTML = cssString;

          var existingElement = document.getElementById(userColoursStyleElementId);
          if (existingElement) {
            existingElement.parentNode.removeChild(existingElement);
          }

          document.head.appendChild(sheet);
        });
    };

    return customStyles;
  });
