'use strict';
angular.module('aom.services')
  .factory('aomUserSrv', function ($q, $rootScope, $timeout, aomDatabaseSrv, aomResourcesSrv, aomDeviceSrv, aomNetworkSrv, aomAuthSrv, aomAPISrv, aomLocationSrv, aomFileSystemSrv, aomCleanSrv, constants, aomLogSrv, aomLoginTaskManagerSrv, aomCustomStylesSrv, aomDevResourcesSrv, aomComponentsSrv) {
    var userService = {};

    userService.offlineLogon = false;
    userService.isFirstOfflineLogon = true;
    userService.logonRequest = {
      Username: aomResourcesSrv.data.lastUser.Username,
      Password: null,
      DeviceInfo: {
        Platform: ionic.Platform.platform(),
        OperatingSystemVersion: window.cordova ? window.device.version : 'Browser',
        AppVersion: constants.common.appVersion,
        MacAddress: aomDeviceSrv.uuid(),
        Manufacturer: aomDeviceSrv.manufacturer(),
        Model: aomDeviceSrv.model()
      },
      AppName: constants.common.appName,
      Coordinates: '',
      failedLoginCount: aomResourcesSrv.data.lastUser.failedLoginCount
    };

    function dbUpdate() {
      var user = angular.copy(userService.logonRequest);
      user.resources = aomResourcesSrv.data.resources;
      user.downloadedLogoLink = aomResourcesSrv.data.lastUser.downloadedLogoLink;
      user.logonRequest = angular.copy(userService.logonRequest);
      ['failedLoginCount', 'failedLogonCount', 'groups'].forEach(function (key) {
        if (user.logonRequest[key]) {
          delete user.logonRequest[key];
        }
      });
      var insertObject = {
        LastUser: {
          id: 1,
          user_json: user
        }
      };
      return aomDatabaseSrv.insertQuery(insertObject, true, false)
        .catch(function (err) {
          aomLogSrv.log({ message: 'Failed to updateLastUser', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
        });
    }

    function dbRead() {
      return aomDatabaseSrv.selectQuery({ LastUser: { user_json: null } })
        .then(function (response) {
          var lastUser = response.rows[0];
          if (lastUser !== undefined) {
            aomResourcesSrv.data.lastUser.Username = lastUser.Username;
            aomResourcesSrv.data.lastUser.groups = lastUser.groups;
            aomResourcesSrv.data.lastUser.Password = lastUser.Password;
            aomResourcesSrv.data.lastUser.failedLoginCount = lastUser.failedLoginCount;
            aomResourcesSrv.data.lastUser.downloadedLogoLink = lastUser.downloadedLogoLink;
            aomResourcesSrv.data.normalizedLogo = lastUser.downloadedLogoLink
              ? aomFileSystemSrv.normalizeURL(lastUser.downloadedLogoLink)
              : constants.common.defaultLogo;
            aomResourcesSrv.data.lastUser.logonRequest = lastUser.logonRequest;
            aomResourcesSrv.data.initilized = true;

            if (lastUser.resources) {
              angular.copy(lastUser.resources, aomResourcesSrv.data.resources);
            }
            aomResourcesSrv.setupApplications();
          }
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Could not read last user', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
        });
    }

    function getResources() {    
      return aomAPISrv.getData(aomAPISrv.data.rootLinks.Settings.href)
        .then(function (response) {
          angular.copy(response.Settings, aomResourcesSrv.data.resources);

          //Removing from resources
          if (aomResourcesSrv.data.MOP) {
            delete aomResourcesSrv.data.resources.SideMenu.Logout;
            delete aomResourcesSrv.data.resources.SideMenu.Databases;
            var indexes = [1, 10, 11, 12, 13, 14, 15, 19, 20];
            var filteredList = aomResourcesSrv.data.resources.Settings.Data.filter(function (item, idx) {
              return indexes.indexOf(idx) === -1;
            });
            aomResourcesSrv.data.resources.Settings.Data = filteredList;
          }

          aomResourcesSrv.data.resources.logoHref = response.links.logo.href;
          return aomAPISrv.getData(aomAPISrv.data.rootLinks.AppSettings.href);
        })
        .then(function (response) {
          aomResourcesSrv.data.resources.appConfig = response.AppSettings;
          //Adding Configs

          //Removing from Configs
          if (aomResourcesSrv.data.MOP) {
            aomResourcesSrv.data.resources.appConfig.ExtraInfo.splice(4, 1);
            aomResourcesSrv.data.resources.appConfig.SortOptions.splice(4, 1);
          }

          aomResourcesSrv.data.resources.appSettingsSaveLink = response.links.saveappsettings.href;
          aomResourcesSrv.data.resources.derived = aomCleanSrv.derivedData(aomResourcesSrv.data.resources.appConfig);

          aomDevResourcesSrv.init();
          aomCustomStylesSrv.init();
          aomResourcesSrv.setupApplications();

        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Failed to get resources', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
          return $q.reject(err);
        });
    }

    function updateLogo(filename) {
      aomResourcesSrv.data.lastUser.downloadedLogoLink = filename ? aomFileSystemSrv.rootApplicationPath + constants.common.logoFolder + '/' + filename : null;
    }

    function getLogo() {
      if (aomResourcesSrv.data.resources.logoHref) {
        var filename = aomResourcesSrv.data.resources.logoHref.split('/').pop();
        if (window.cordova && filename.length > 0) {
          return aomFileSystemSrv.logoExists(filename, constants.common.logoFolder)
            .then(function (file) {
              updateLogo(file.name);
              return $q.when();
            })
            .catch(function () {
              return aomAPISrv.getFile(aomResourcesSrv.data.resources.logoHref)
                .then(function (response) {
                  const contentType = response.headers('Content-Type');
                  const extension = aomComponentsSrv.getExtensionForContentType(contentType);
                  const name = filename + extension;
                  updateLogo(name);
                  return aomFileSystemSrv.write(name, response, constants.common.logoFolder);
                })
                .catch(function (err) {
                  aomLogSrv.log({ message: 'Failed to retreive logo', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
                  return $q.when(filename);
                });
            });
        } else {
          return $q.when(filename);
        }
      } else {
        updateLogo(null);
        return $q.when();
      }
    }

    function getUserDataFromServer() {
      return getResources()
        .then(function () {
          return aomAPISrv.getData(aomAPISrv.data.rootLinks[constants.common.applicationResourceName].href);
        })
        .then(function (response) {
          userService.logonRequest.groups = aomCleanSrv.cleanGroups(response.Applications);
          userService.logonRequest.failedLoginCount = 0;
          return updateLastUser();
        })
        .then(function () {
          return dbRead();
        })
        .then(function () {
          if (!aomResourcesSrv.data.OEM && !aomResourcesSrv.data.MOP) {
            getLogo()
              .then(function () {
                return userService.updateLastUser();
              })
              .then(function () {
                return dbRead();
              });
          }
          return $q.when();
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Error getting user data from server', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
          return $q.reject({ message: 'Error getting user data from server', error: err });
        });
    }

    function updateLastUser() {
      if (aomResourcesSrv.data.lastUser !== undefined && aomResourcesSrv.data.lastUser !== null && aomResourcesSrv.data.lastUser.Username !== userService.logonRequest.Username) {
        return aomDeviceSrv.clean()
          .then(function () {
            return dbUpdate();
          });
      } else {
        return dbUpdate();
      }
    }
    function callLogon(location) {
      userService.logonRequest.Coordinates = location.latitude + ',' + location.longitude;
      return aomAuthSrv.logon(userService.logonRequest)
        .catch(function (err) {
          if (err.status === 500 || err.status === -1) {
            return $q.reject(err);
          } else {
            return $q.reject({ loginError: aomResourcesSrv.data.resources.Errors.LoginFail + ' ' + err.data.Description });
          }
        });
    }

    function onlineLogon() {
      return aomLocationSrv.getAccurateCurrentLocation()
        .then(function () {
          return callLogon(aomLocationSrv.data.lastLocationForServer);
        })
        .then(function () {
          userService.logonRequest.failedLoginCount = 0;
          aomLoginTaskManagerSrv.data.runTasks = true;
          userService.offlineLogon = false;
          return getUserDataFromServer();
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Failed to login', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.info);
          if (err.loginError) {
            return $q.reject(err);
          } else {
            return offlineLogon();
          }
        });
    }

    // When the system comes online after an offline logon we need to wait
    // for the root links to recover before we attempt an online logon.
    // This will enable all of the api calls to then be correctly authenticated.
    function waitForRootLinks() {
      $timeout(function () {
        return checkRootLinks();
      }, 1000);
    }

    function checkRootLinks() {
      if (aomNetworkSrv.onlineWithRootLinks()) {
        onlineLogon();
      } else {
        waitForRootLinks();
      }
    }

    function offlineLogon() {
      if (aomResourcesSrv.data.lastUser.Username !== '') {
        var usernameValid = userService.logonRequest.Username === aomResourcesSrv.data.lastUser.Username,
          passwordValid = userService.logonRequest.Password === aomResourcesSrv.data.lastUser.Password;

        if (usernameValid && passwordValid) {

          // We need to watch for the app coming on line, from on offline log state
          // Ensure we don't watch for the network status event multiple times
          userService.offlineLogon = true;
          if (userService.isFirstOfflineLogon) {
            userService.isFirstOfflineLogon = false;
            $rootScope.$on('$aomNetworkSrv:online', function() {
              if (userService.offlineLogon) {
                console.log('Online network status detected from offline logon');
                checkRootLinks();
              }             
            });  
          }

          return $q.when();

        } else {
          return incrementAndCheckFailedLogonCounter(aomResourcesSrv.data.resources.Errors.LoginFail);
        }
      } else {
        return $q.reject({ loginError: aomResourcesSrv.data.resources.Errors.LoginFail + ' ' + aomResourcesSrv.data.resources.Errors.NoInternet });
      }
    }

    function incrementAndCheckFailedLogonCounter(errorMessage) {
      var errorResponse = {
        loginError: aomResourcesSrv.data.resources.Errors.LoginFail + ' ' + errorMessage
      };
      userService.logonRequest.failedLoginCount += 1;

      if (userService.logonRequest.failedLoginCount > aomResourcesSrv.data.resources.Common.LoginFailCounter) {
        errorResponse.loginError += ' ' + aomResourcesSrv.data.resources.Errors.DataReset;
        return aomDeviceSrv.cleanLogo()
          .then(function () {
            return aomDeviceSrv.clean();
          })
          .then(function () {
            return $q.reject(errorResponse);
          });
      }
      return $q.reject(errorResponse);
    }

    userService.updateLastUser = function () {
      return dbUpdate();
    };

    userService.init = function () {
      return dbRead();
    };

    userService.logon = function () {
      if (aomNetworkSrv.onlineWithRootLinks()) {
        return onlineLogon();
      } else {
        return offlineLogon();
      }
    };

    return userService;
  });