﻿'use strict';
angular.module('aom.directives')
  .directive('aomCanvas', function ($timeout, $ionicPopup, $ionicLoading, aomResourcesSrv, aomJobSrv, aomDrawingSrv, constants, aomMediaSrv, aomBackSrv, aomComponentsSrv, aomPopupSrv, aomTextScreenSrv) {
    return {
      scope: {
        back: '&'
      },
      controller: function ($scope) {
        var ctx = document.getElementById('c').getContext('2d'),
          imageUrl = null,
          fillColorMode = false;

        function showSaveMessagePopup() {
          var saveMessage = $scope.data.field.Type === constants.fieldTypes.signature ? $scope.data.resources.ImageEditor.SaveSignatureMessage : $scope.data.resources.ImageEditor.SaveMessage;

          aomPopupSrv.wrapper({
            id: 'imageEditorSave',
            title: $scope.data.resources.ImageEditor.Save,
            template: saveMessage,
            cssClass: 'large-popup',
            buttons: [
              {
                text: $scope.data.resources.Common.Cancel,
                type: 'button-stable button-outline',
              },
              {
                text: $scope.data.resources.Common.No,
                type: 'button-stable button-outline',
                tapMethod: function () {
                  exitScreen();
                }
              },
              {
                text: $scope.data.resources.Common.Yes,
                type: 'button-positive',
                tapMethod: function () {
                  $ionicLoading.show().then(function () {
                    aomDrawingSrv.saveCanvas($scope.data.field, $scope.data.comment).finally(function () {
                      $ionicLoading.hide().then(function () {
                        exitScreen();
                      });
                    });
                  });
                }
              }
            ]
          });
        }

        function exitScreen() {
          aomBackSrv.exit(true);
          $scope.back();
        }

        function goBack() {
          aomDrawingSrv.changesMade() || $scope.data.comment !== null ? showSaveMessagePopup() : exitScreen();
        }

        function showMandatorySignatureMessage() {
          aomPopupSrv.wrapper({
            id: 'imageEditorMandatory',
            title: $scope.data.resources.ImageEditor.Mandatory,
            template: $scope.data.resources.ImageEditor.MandatoryMessage,
            cssClass: 'large-popup',
            buttons: [
              {
                text: $scope.data.resources.Common.No,
                type: 'button-stable button-outline'
              },
              {
                text: $scope.data.resources.Common.Yes,
                type: 'button-positive',
                tapMethod: function () {
                  goBack();
                }
              }
            ]
          });
        }

        function backClicked() {
          var mandatorySignatureRequirementFulfilled = $scope.data.field.Type === constants.fieldTypes.signature && $scope.data.field.Options.IsRequired ?
            (aomDrawingSrv.changesMade() || $scope.data.field.Attachments.length > 0) && $scope.data.comment !== null && $scope.data.comment.length > 0 : true;

          mandatorySignatureRequirementFulfilled ? goBack() : showMandatorySignatureMessage();
        }

        $scope.backTapped = function () {
          backClicked();
        };

        function clear(field) {
          $scope.data.comment = null;
          aomDrawingSrv.clearClick(field);
        }


        aomDrawingSrv.resetDrawingSurface();

        $scope.data = {
          field: aomJobSrv.data.selectedField,
          resources: aomResourcesSrv.data.resources,
          drawingServiceData: aomDrawingSrv.data,
          serviceProps: {
            drawingColour: '#000000',
            lineWidth: 10
          },
          colors: aomResourcesSrv.data.resources.ImageEditor.Colors,
          scaleFactor: 100,
          actionDescription: aomResourcesSrv.data.resources.ImageEditor.Select,
          actionMode: 'draw',
          comment: null,
          showCanvas: true,
          menuTree: [],
          previousActionMode: null,
          tool: null
        };

        if (aomJobSrv.data.selectedField.Attachments !== undefined && aomJobSrv.data.selectedField.Attachments.length > 0) {
          $scope.data.comment = aomJobSrv.data.selectedField.Attachments[0].Comment;
        }

        $scope.fieldTypes = constants.fieldTypes;

        if ($scope.data.field.Type === constants.fieldTypes.signature) {
          $scope.data.serviceProps.lineWidth = 2;
        }

        if ($scope.data.field.Parameters.Type !== '' && $scope.data.field.filename !== '') {
          if ($scope.data.field.Attachments.length > 0) {
            // Can't access our images on serve due to auth.
            imageUrl = window.cordova ? $scope.data.field.Attachments[0].filename : 'https://vignette.wikia.nocookie.net/disney/images/f/f7/Olaf_transparent_pose.png/revision/latest?cb=20160301215157';
          }
        }

        aomDrawingSrv.init(ctx, imageUrl, $scope.data.serviceProps, $scope.data.field);
        aomBackSrv.init(backClicked, 201);

        $scope.clearClick = function (field) {
          if (field.MaintainBackgroundImageOnClear) {
            clear(field);
          } else {
            var confirmPopup = $ionicPopup.confirm({
              template: aomResourcesSrv.data.resources.ImageEditor.ClearWarning
            });
            confirmPopup.then(function (res) {
              if (res) {
                clear(field);
              }
            });
          }
        };

        $scope.selectLineWidth = function () {
          aomDrawingSrv.data.selectedObject === null ?
            aomDrawingSrv.selectLineWidth($scope.data.serviceProps.lineWidth) :
            aomDrawingSrv.setObjectLineWidth($scope.data.serviceProps.lineWidth);
        };

        $scope.colorSelected = function (selectedColor) {
          $scope.data.serviceProps.drawingColour = selectedColor;
          aomDrawingSrv.data.selectedObject === null ?
            aomDrawingSrv.colorSelected(selectedColor) :
            aomDrawingSrv.setObjectColor(selectedColor, fillColorMode);
        };

        $scope.scaleObject = function () {
          aomDrawingSrv.scale($scope.data.scaleFactor);
        };

        $scope.undo = aomDrawingSrv.undo;
        $scope.redo = aomDrawingSrv.redo;
        $scope.remove = aomDrawingSrv.remove;
        $scope.scale = aomDrawingSrv.scale;
        $scope.addCircle = aomDrawingSrv.addCircle;
        $scope.addSquare = aomDrawingSrv.addSquare;
        $scope.addTriangle = aomDrawingSrv.addTriangle;

        $scope.addText = function () {
          $scope.selectObject = { Data: aomDrawingSrv.data.selectedObject !== null ? aomDrawingSrv.data.selectedObject.text : '' };
          aomTextScreenSrv.show($scope);
        };

        // TODO Change when the text service is refactored
        $scope.change = function () {
          aomDrawingSrv.data.selectedObject === null ?
            aomDrawingSrv.addText($scope.selectObject.Data) :
            aomDrawingSrv.updateText($scope.selectObject.Data);
        };

        function setSelectMode() {
          $scope.data.actionMode = 'select';
          $scope.data.actionDescription = aomResourcesSrv.data.resources.ImageEditor.Drawing;
          aomDrawingSrv.setMode(false);
        }

        function setDrawingMode() {
          $scope.data.actionMode = 'draw';
          $scope.data.actionDescription = aomResourcesSrv.data.resources.ImageEditor.Select;
          aomDrawingSrv.setMode(true);
        }

        function backToPrevious() {
          $scope.data.previousActionMode === 'draw' ? setDrawingMode() : setSelectMode();
        }

        function actionModeBack() {
          if (aomDrawingSrv.data.deselectMethod !== null) {
            aomDrawingSrv.data.deselectMethod();
          }
          aomDrawingSrv.data.deselectMethod = null;
          aomDrawingSrv.data.selectMethod = null;
          backToPrevious();
        }

        function setActionMode(mode) {
          $scope.data.previousActionMode = angular.copy($scope.data.actionMode);
          $scope.data.actionMode = mode;
          $scope.data.actionDescription = aomResourcesSrv.data.resources.Common.Close;
        }

        $scope.controlAction = function () {
          switch ($scope.data.actionMode) {
            case 'draw':
              setSelectMode();
              break;
            case 'select':
              setDrawingMode();
              break;
            default:
              actionModeBack();
              break;
          }
        };

        $scope.showColours = function (fill) {
          setActionMode('colours');
          fillColorMode = fill;
          aomDrawingSrv.data.deselectMethod = function () {
            aomDrawingSrv.forceAddToHistory();
            backToPrevious();
          };
        };

        $scope.showWidth = function () {
          setActionMode('width');
        };

        $scope.showScale = function () {
          if (aomDrawingSrv.data.selectedObject !== null) {
            $scope.data.scaleFactor = 10;
            setActionMode('scale');
            aomDrawingSrv.data.deselectMethod = function () {
              aomDrawingSrv.forceAddToHistory();
              backToPrevious();
            };
            aomDrawingSrv.data.selectMethod = $scope.showScale;
          }
        };

        $scope.showShapes = function () {
          setActionMode('shapes');
        };

        $scope.addPhoto = function () {
          var params = { source: { photo: true, video: false, gallery: true }, orientation: 'landscape' };
          var file = null;

          $timeout(function () {
            aomMediaSrv.showMediaOptions(params)
              .then(function (media) {
                file = media.file;
                if (file.name.length > 0) {
                  aomDrawingSrv.isPhotoTaken = true;
                  aomDrawingSrv.addImage(file.name);
                  setSelectMode();
                }
              });
          });
        };

        $scope.showCanvas = function () {
          $timeout(function () {
            $scope.data.showCanvas = true;
          }, 200);
        };

        $scope.hideCanvas = function () {
          $scope.data.showCanvas = false;
        };

        function destroy() {
          aomComponentsSrv.unlockOrientation();
        }

        $scope.$on('$destroy', function () {
          destroy();
        });
      },
      templateUrl: 'App/Components/ImageEditor/imageEditor.html'
    };
  });